"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutputFile = void 0;
const lodash_1 = __importDefault(require("lodash"));
class OutputFile {
    constructor(path) {
        this.path = path;
        this.functions = {
            found: 0,
            hit: 0,
            details: [],
        };
        this.lines = {
            found: 0,
            hit: 0,
            details: [],
        };
        this.branches = {
            found: 0,
            hit: 0,
            details: [],
        };
    }
    addFunction(func) {
        var functions = this.functions;
        functions.found++;
        functions.hit += func.hit;
        functions.details.push(func);
        return this;
    }
    addLine(line) {
        var lines = this.lines;
        lines.found++;
        lines.hit += line.hit;
        lines.details.push(line);
        return this;
    }
    addBranch(branch) {
        var branches = this.branches;
        branches.found++;
        branches.hit += branch.taken;
        branches.details.push(branch);
        return this;
    }
    toString() {
        var output = [];
        // Header
        output.push("TN:");
        output.push("SF:" + this.path);
        // Functions
        lodash_1.default.each(this.functions.details, function (func) {
            output.push("FN:" + func.line + "," + func.name);
        });
        output.push("FNF:" + this.functions.found);
        output.push("FNH:" + this.functions.hit);
        lodash_1.default.each(this.functions.details, function (func) {
            output.push("FNDA:" + func.hit + "," + func.name);
        });
        // Lines
        lodash_1.default.each(this.lines.details, function (line) {
            output.push("DA:" + line.line + "," + line.hit);
        });
        output.push("LF:" + this.lines.found);
        output.push("LH:" + this.lines.hit);
        // Branches
        lodash_1.default.each(this.branches.details, function (branch) {
            output.push("BRDA:" +
                branch.line +
                "," +
                branch.block +
                "," +
                branch.branch +
                "," +
                branch.taken);
        });
        output.push("BRF:" + this.branches.found);
        output.push("BRH:" + this.branches.hit);
        // Footer
        output.push("end_of_record");
        return output.join("\n");
    }
}
exports.OutputFile = OutputFile;
