"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLcovData = exports.getData = exports.getTransformedFiles = exports.getOutputLcov = exports.writeLcov = exports.getLcov = void 0;
const convert_source_map_1 = __importDefault(require("convert-source-map"));
const promises_1 = __importDefault(require("fs/promises"));
const lcov_parse_1 = __importDefault(require("lcov-parse"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const source_map_1 = require("source-map");
const util_1 = require("util");
const file_2 = require("./file");
function getLcov(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let file = yield getTransformedFiles(options);
        return getOutputLcov(file, options.sourceDir);
    });
}
exports.getLcov = getLcov;
function writeLcov(outputFile, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = yield getLcov(options);
        return yield promises_1.default.writeFile(outputFile, data);
    });
}
exports.writeLcov = writeLcov;
function getOutputLcov(files, sourceDir) {
    return __awaiter(this, void 0, void 0, function* () {
        sourceDir = sourceDir || process.cwd();
        const resolver = yield Promise.all(lodash_1.default.map(files, function (file) {
            // Check if used package tool like webpack or otherwise
            let match = file.path.match(/^\.\/([a-z]*?):\/\/(.*)/i);
            if (match != null) {
                let protocol = match[1], bundlerPath = match[2];
                if (protocol != "file" && bundlerPath[0] === "/") {
                    bundlerPath = bundlerPath.substring(1);
                }
                file.path = bundlerPath;
            }
            let it = new Promise(function (resolve) {
                return __awaiter(this, void 0, void 0, function* () {
                    let exists = yield promises_1.default.stat(path_1.default.resolve(sourceDir, file.path));
                    if (exists != null) {
                        resolve(file);
                    }
                });
            });
            return it;
        })).then((it) => lodash_1.default.filter(it));
        var output = [];
        lodash_1.default.each(resolver, function (file_1) {
            output.push(file_1.toString());
        });
        return output.join("\n");
    });
}
exports.getOutputLcov = getOutputLcov;
function getTransformedFiles(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = yield getData(options);
        return lodash_1.default.chain(data.lcov)
            .map(function (lcov, key) {
            var sourcemap = data.sourcemap[key];
            if (!sourcemap) {
                throw new Error("Missing sourcemap: " + key);
            }
            return transformLcovMap(lcov, sourcemap, options.sourceDir);
        })
            .map(function (group) {
            return lodash_1.default.values(group);
        })
            .flatten()
            .value();
    });
}
exports.getTransformedFiles = getTransformedFiles;
function transformLcovMap(lcov, sourcemap, sourceDir) {
    let consumer = sourcemap.consumer;
    let files = {};
    let getFile;
    if (!consumer.sourceRoot) {
        // sourcemaps without sourceRoot will be relative to dist
        getFile = function (source) {
            var fn = "." +
                path_1.default.sep +
                path_1.default.relative(sourceDir, path_1.default.join(path_1.default.dirname(sourcemap.path), source.source));
            return (files[fn] = files[fn] || new file_2.OutputFile(fn));
        };
    }
    else {
        getFile = function (source) {
            var fn = source.source.replace(consumer.sourceRoot, "./");
            return (files[fn] = files[fn] || new file_2.OutputFile(fn));
        };
    }
    lodash_1.default.each(lcov.functions.details, function (func) {
        var source = consumer.originalPositionFor({
            line: func.line,
            column: 0,
            bias: source_map_1.SourceMapConsumer.LEAST_UPPER_BOUND,
        });
        // Can't find it in source map, fuhgeddaboudit
        if (!source || !source.source) {
            return;
        }
        getFile(source).addFunction({
            name: func.name,
            line: source.line,
            hit: func.hit,
        });
    });
    lodash_1.default.each(lcov.lines.details, function (line) {
        var source = consumer.originalPositionFor({
            line: line.line,
            column: 0,
            bias: source_map_1.SourceMapConsumer.LEAST_UPPER_BOUND,
        });
        // Can't find it in source map, fuhgeddaboudit
        if (!source || !source.source) {
            return;
        }
        getFile(source).addLine({
            line: source.line,
            hit: line.hit,
        });
    });
    lodash_1.default.each(lcov.branches.details, function (branch) {
        var source = consumer.originalPositionFor({
            line: branch.line,
            column: 0,
            bias: source_map_1.SourceMapConsumer.LEAST_UPPER_BOUND,
        });
        // Can't find it in source map, fuhgeddaboudit
        if (!source || !source.source) {
            return;
        }
        getFile(source).addBranch({
            block: branch.block,
            line: source.line,
            branch: branch.branch,
            taken: branch.taken,
        });
    });
    return files;
}
function getData(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let lcov = yield getLcovData(options.lcov);
        let sourcemap = yield getSourcemapsData(lcov, options);
        return {
            lcov,
            sourcemap,
        };
    });
}
exports.getData = getData;
function getSourcemapsData(lcov, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let sources = Object.entries(lcov).map(([lcovpath, _]) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            let sourcepath = (_b = (_a = options.sourcemaps) === null || _a === void 0 ? void 0 : _a.call(options, lcovpath)) !== null && _b !== void 0 ? _b : `${lcovpath}.map`;
            let file = yield promises_1.default.readFile(sourcepath);
            let content = (() => {
                var _a;
                if (path_1.default.extname(sourcepath) === ".map") {
                    return file.toString();
                }
                else {
                    return (_a = convert_source_map_1.default.fromSource(file.toString())) === null || _a === void 0 ? void 0 : _a.toObject();
                }
            })();
            return [
                lcovpath,
                {
                    path: sourcepath,
                    consumer: yield source_map_1.SourceMapConsumer.fromSourceMap(content),
                },
            ];
        }));
        let objects = yield Promise.all(sources);
        return Object.fromEntries(objects);
    });
}
const lcovParse = (0, util_1.promisify)(lcov_parse_1.default);
function getLcovData(lcov) {
    return __awaiter(this, void 0, void 0, function* () {
        let file = yield lcovParse(lcov);
        let it = file === null || file === void 0 ? void 0 : file.map((item) => {
            return [item.file, item];
        });
        return Object.fromEntries(it !== null && it !== void 0 ? it : []);
    });
}
exports.getLcovData = getLcovData;
